"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DefaultSoftCrudRepository = void 0;
const repository_1 = require("@loopback/repository");
const rest_1 = require("@loopback/rest");
class DefaultSoftCrudRepository extends repository_1.DefaultTransactionalRepository {
    constructor(entityClass, dataSource, currentUser) {
        super(entityClass, dataSource);
        this.currentUser = currentUser;
    }
    find(filter, options) {
        var _a;
        // Filter out soft deleted entries
        if ((filter === null || filter === void 0 ? void 0 : filter.where) &&
            filter.where.and &&
            filter.where.and.length > 0) {
            filter.where.and.push({
                deleted: false,
            });
        }
        else if ((filter === null || filter === void 0 ? void 0 : filter.where) &&
            filter.where.or &&
            filter.where.or.length > 0) {
            filter.where.and = [];
            filter.where.and.push({
                deleted: false,
            }, {
                or: filter.where.or,
            });
        }
        else {
            filter = filter !== null && filter !== void 0 ? filter : {};
            filter.where = (_a = filter.where) !== null && _a !== void 0 ? _a : {};
            filter.where.deleted = false;
        }
        // Now call super
        return super.find(filter, options);
    }
    //find all enteries even with soft deleted records
    findAll(filter, options) {
        return super.find(filter, options);
    }
    findOne(filter, options) {
        var _a;
        // Filter out soft deleted entries
        if ((filter === null || filter === void 0 ? void 0 : filter.where) &&
            filter.where.and &&
            filter.where.and.length > 0) {
            filter.where.and.push({
                deleted: false,
            });
        }
        else if ((filter === null || filter === void 0 ? void 0 : filter.where) &&
            filter.where.or &&
            filter.where.or.length > 0) {
            filter.where.and = [];
            filter.where.and.push({
                deleted: false,
            }, {
                or: filter.where.or,
            });
        }
        else {
            filter = filter !== null && filter !== void 0 ? filter : {};
            filter.where = (_a = filter.where) !== null && _a !== void 0 ? _a : {};
            filter.where.deleted = false;
        }
        // Now call super
        return super.findOne(filter, options);
    }
    //findOne() including soft deleted entry
    findOneIncludeSoftDelete(filter, options) {
        return super.findOne(filter, options);
    }
    async findById(id, filter, options) {
        // Filter out soft deleted entries
        // Filter out soft deleted entries
        if ((filter === null || filter === void 0 ? void 0 : filter.where) &&
            filter.where.and &&
            filter.where.and.length > 0) {
            filter.where.and.push({
                deleted: false,
                id: id,
            });
        }
        else if ((filter === null || filter === void 0 ? void 0 : filter.where) &&
            filter.where.or &&
            filter.where.or.length > 0) {
            filter.where = {
                and: [
                    {
                        deleted: false,
                        id: id,
                    },
                    {
                        or: filter.where.or,
                    },
                ],
            };
        }
        else {
            filter = filter !== null && filter !== void 0 ? filter : {};
            filter.where = {
                deleted: false,
                id: id,
            };
        }
        //As parent method findById have filter: FilterExcludingWhere<T>
        //so we need add check here.
        const entityToRemove = await super.findOne(filter, options);
        if (entityToRemove) {
            // Now call super
            return super.findById(id, filter, options);
        }
        else {
            throw new rest_1.HttpErrors.NotFound("EntityNotFound" /* ErrorKeys.EntityNotFound */);
        }
    }
    //find by Id including soft deleted record
    findByIdIncludeSoftDelete(id, filter, options) {
        return super.findById(id, filter, options);
    }
    updateAll(data, where, options) {
        // Filter out soft deleted entries
        if (where &&
            where.and &&
            where.and.length > 0) {
            where.and.push({
                deleted: false,
            });
        }
        else if (where &&
            where.or &&
            where.or.length > 0) {
            where.and = [];
            where.and.push({
                deleted: false,
            }, {
                or: where.or,
            });
        }
        else {
            where = where !== null && where !== void 0 ? where : {};
            where.deleted = false;
        }
        // Now call super
        return super.updateAll(data, where, options);
    }
    count(where, options) {
        // Filter out soft deleted entries
        if (where &&
            where.and &&
            where.and.length > 0) {
            where.and.push({
                deleted: false,
            });
        }
        else if (where &&
            where.or &&
            where.or.length > 0) {
            where.and = [];
            where.and.push({
                deleted: false,
            }, {
                or: where.or,
            });
        }
        else {
            where = where !== null && where !== void 0 ? where : {};
            where.deleted = false;
        }
        // Now call super
        return super.count(where, options);
    }
    async delete(entity, options) {
        // Do soft delete, no hard delete allowed
        entity.deleted = true;
        entity.deletedOn = new Date();
        entity.deletedBy = await this.getUserId();
        return super.update(entity, options);
    }
    async deleteAll(where, options) {
        // Do soft delete, no hard delete allowed
        return this.updateAll({
            deleted: true,
            deletedOn: new Date(),
            deletedBy: await this.getUserId(),
        }, where, options);
    }
    async deleteById(id, options) {
        // Do soft delete, no hard delete allowed
        return super.updateById(id, {
            deleted: true,
            deletedOn: new Date(),
            deletedBy: await this.getUserId(),
        }, options);
    }
    /**
     * Method to perform hard delete of entries. Take caution.
     * @param entity
     * @param options
     */
    deleteHard(entity, options) {
        // Do hard delete
        return super.deleteById(entity.getId(), options);
    }
    /**
     * Method to perform hard delete of entries. Take caution.
     * @param entity
     * @param options
     */
    deleteAllHard(where, options) {
        // Do hard delete
        return super.deleteAll(where, options);
    }
    /**
     * Method to perform hard delete of entries. Take caution.
     * @param entity
     * @param options
     */
    deleteByIdHard(id, options) {
        // Do hard delete
        return super.deleteById(id, options);
    }
    async getUserId(options) {
        var _a, _b, _c, _d;
        if (!this.currentUser) {
            return undefined;
        }
        if (!this.currentUser || !this.currentUser.id) {
            return undefined;
        }
        const uid = (_d = (_b = (_a = this.currentUser) === null || _a === void 0 ? void 0 : _a.userTenantId) !== null && _b !== void 0 ? _b : (_c = this.currentUser) === null || _c === void 0 ? void 0 : _c.id) !== null && _d !== void 0 ? _d : '';
        return uid.toString();
    }
}
exports.DefaultSoftCrudRepository = DefaultSoftCrudRepository;
//# sourceMappingURL=default-soft-crud.repository.base.js.map